/* implement a singly linked list of handles.
	93/12/18 aih - uses LLPtrLib
	93/12/01 aih - uses memory manager again
	93/10/31 aih - uses calloc/free rather than memory manager
	93/10/28 aih - created
*/

#include <string.h>
#include "LinkedListLib.h"
#include "MemoryLib.h"

/* true if list is valid */
Boolean LListValid(LListType head)
{
	return(PtrValidSize(head, sizeof(LListStruct)));
}

/* return a new empty list */
LListType LListBegin(void)
{
	return(NULL);
}

/* dispose of all of the items in the list */
void LListEnd(LListType head)
{
	require(! head || LListValid(head));
	while (head) {
		LListType item = head;
		head = LLPNext(head);
		PtrEnd(item);
	}
	ensure(! LListValid(head));	
}

/* return the item's data */
void *LListData(LListType item)
{
	require(LListValid(item));
	return(item + 1);
}

/* return item's data size */
size_t LListDataSize(LListType item)
{
	require(LListValid(item));
	return(PtrSize(item) - sizeof(LListStruct));
}

/* return the next item in the list */
LListType LListNext(LListType item)
{
	return(LLPNext(item));
}

/* return item matching data */
LListType LListFind(LListType head, const void *data,
	int (*compare)(const void *data1, const void *data2))
{
	require(! head || LListValid(head));
	while (head) {
		int cmp = compare(data, LListData(head));
		if (cmp == 0)
			break;
		head = LLPNext(head);
	}
	return(head);
}

/* insert item at head of list, return new head */
LListType LListInsert(LListType head, LListType item)
{
	return(LLPInsert(head, item));
}

/* delete item from list, return new head */
LListType LListDelete(LListType head, LListType item)
{
	return(LLPDelete(head, item));
}

/* move item to head of list */
LListType LListMoveToFront(LListType head, LListType item)
{
	return(LLPInsert(LLPDelete(head, item), item));
}

