#pragma once

#include <stddef.h>

/* a node in the tree */
typedef struct NodeStruct {
	struct NodeStruct *parent;	/* parent of node */
	struct NodeStruct *left;	/* left child */
	struct NodeStruct *right;	/* right child */
	/* ... more data ... */
} NodeType, *NodePtr, **NodeHandle;

/* a tree */
typedef struct {
	NodePtr root;		/* root node of tree */
	size_t size;		/* number of nodes in tree */
	Boolean splay;		/* if true, splaying is enabled */
} TreeType, *TreePtr, **TreeHandle;

typedef int (*TreeCompareType)(const void *key, const NodePtr node);
typedef void (*TreeVisitType)(const NodePtr node, void *data);

Boolean TreeValid(TreePtr tree);

TreePtr TreeBegin(Boolean splay);
void TreeEnd(TreePtr tree, TreeVisitType dispose);

size_t TreeSize(TreePtr tree);
NodePtr TreeRoot(TreePtr tree);

void TreeSplay(TreePtr tree, NodePtr node);
NodePtr TreeFindAt(TreePtr tree, const void *key,
	TreeCompareType compare, NodePtr *pparent, int *pcmp);
void TreeInsertAt(TreePtr tree, NodePtr node, NodePtr parent, int cmp);
void TreeInsert(TreePtr tree, NodePtr node, const void *key,
	TreeCompareType compare);
NodePtr TreeAccess(TreePtr tree, const void *key, TreeCompareType compare);
void TreeDelete(TreePtr tree, NodePtr node);

void TreeInOrder(NodePtr node, TreeVisitType visit, void *data);
void TreePreOrder(NodePtr node, TreeVisitType visit, void *data);
void TreePostOrder(NodePtr node, TreeVisitType visit, void *data);

size_t TreeHeight(NodePtr node, size_t height);
